Attribute VB_Name = "modGeneral"
'@desc      This module contains generic functions used in various projects.
'@author    Balaji Srinivasan
'@version   1.0

Option Explicit
'API Functions for Windows INI file manipulation
Public Declare Function GetPrivateProfileString Lib "kernel32" Alias "GetPrivateProfileStringA" (ByVal lpSectionName As String, ByVal lpKeyName As Any, ByVal lpDefault As String, ByVal lpReturnedString As String, ByVal nSize As Long, ByVal lpFileName As String) As Long
Public Declare Function WritePrivateProfileString Lib "kernel32" Alias "WritePrivateProfileStringA" (ByVal lpSectionName As String, ByVal lpKeyName As Any, ByVal lpString As Any, ByVal lpFileName As String) As Long

Const cKeyValueSeperator = "="
Const cPairSeperator = "::"

'Function to launch the default application with a file, based on the
'Windows mimetype settings in the registry
Public Declare Function ShellExecute _
                            Lib "shell32.dll" _
                            Alias "ShellExecuteA" ( _
                            ByVal hwnd As Long, _
                            ByVal lpOperation As String, _
                            ByVal lpFile As String, _
                            ByVal lpParameters As String, _
                            ByVal lpDirectory As String, _
                            ByVal nShowCmd As Long) _
                            As Long



'@desc      This is a generic function to do mass replace, in a batch process,
'           within a string.
'@param     cInputString    The input string
'@param     cOutputString   the string which will contain the changed values
'@param     oReplaceTags    The object which contains the list of key value pairs which need to be replaced
'@usage     Create a Settings object, and add all the strings to be replaced.
'           Put the string to be replaced in key, and the new string in value.
Public Function SimpleParse(ByVal cInputString As String, _
                    ByRef cOutputString As String, _
                    oReplaceTags As Settings)

Dim oTag As Setting
For Each oTag In oReplaceTags
     cInputString = Replace(cInputString, oTag.Key, oTag.Value)
Next

cOutputString = cInputString
End Function

Public Sub ShowMessage(cMessage As String)
'frmTest..AddItem cMessage
End Sub

'@desc      This function finds a tag within a string, and returns the strings
'           within the tag, before the tag was found, and after the tag was found.
'           It also removes the tags themselves.
'           This also takes care of cases with specific tags as a recurse tag,
'           e.g. [TOPIC_RECURSE], but this is not a recursive function.
'           It has not been tested with nested tags yet!
'@param     cSourceString   The input string
'@param     cSearchTag      The starting tag to be searched e.g. <body>
'@param     cEndSearchTag   The ending tag to be searched e.g. </body>
'@param     cRecurseTag     The specific recurse tag to be searched.
'@param     cTop            The top part of the string, before the beginning of the tag, is returned in this
'@param     cBottom         The bottom part of the string, after the end of the tag, is returned in this
'@param     cTemplate       The contents of the tag is returned in this
Public Function ReadLoopTemplate(cSourceString As String, _
                            cSearchTag As String, _
                            cEndSearchTag As String, _
                            cRecurseTag As String, _
                            ByRef cTop As String, _
                            ByRef cBottom As String, _
                            ByRef cTemplate As String)


'Dim cTopicBuffer As String
Dim iLastPos As Long

cTemplate = ""

iLastPos = 1
Do While True
    'cTopicBuffer = ""
    Dim cTmpTop As String
    Dim cMiddle As String
    Dim cTmpBottom As String
    Dim iTmpLoopPosition As Long
    
    Dim iPos As Long, iPosEnd As Long
    iPos = InStr(iLastPos, cSourceString, cSearchTag)
    If iPos = 0 Then
        Exit Do
    End If
    cTmpTop = Mid(cSourceString, 1, iPos - 1)
    
    iPosEnd = InStr(iLastPos, cSourceString, cEndSearchTag)
    
    cMiddle = Mid(cSourceString, iPos + Len(cSearchTag), iPosEnd - iPos - Len(cEndSearchTag))
    cTmpBottom = Mid(cSourceString, iPosEnd + Len(cEndSearchTag))
        
    iTmpLoopPosition = InStr(cMiddle, cRecurseTag)
    
    cTop = cTmpTop
    cTemplate = cMiddle
    cBottom = cTmpBottom
    
'*********DEBUG********
    Exit Do
'*********/DEBUG********
    iLastPos = iPosEnd + Len(cEndSearchTag)
Loop

'[NOTE]
'This returns only the last(or first?) block currently!!!

End Function

'@desc      This function is a wrapper around the GetPrivateProfileString API call.
'           It reads a value from an INI file and returns that value.
'@param     cIniFile        The INI file from which to read the settings
'@param     cSection        The section of the ini file from which the setting will be retrived
'@param     cKey            The key to search for
'@param     cDefaultValue   The default value to be returned in case no value is found
'@return    GetIniValue     Returns the value if found, or the passed default value
Public Function GetIniValue(cIniFile As String, _
                        cSection As String, _
                        cKey As String, _
                        cDefaultValue As String) As String
Dim iBytes As Long
Const iMaxSize = 32767
Dim cKeyValue As String

cKeyValue = Space(iMaxSize)
iBytes = GetPrivateProfileString(cSection, cKey, cDefaultValue, cKeyValue, iMaxSize, cIniFile)
cKeyValue = Mid$(cKeyValue, 1, (InStr(1, cKeyValue, Chr(0))) - 1)
cKeyValue = Trim(cKeyValue)

GetIniValue = cKeyValue
End Function

Public Function SetIniValue(cIniFile As String, _
                        cSection As String, _
                        cKey As String, _
                        cValue As String) As Long
Dim iRetval As Long
'Const iMaxSize = 32767
'Dim cKeyValue As String

'cKeyValue = Space(iMaxSize)
iRetval = WritePrivateProfileString(cSection, cKey, cValue, cIniFile)
'cKeyValue = Mid$(cKeyValue, 1, (InStr(1, cKeyValue, Chr(0))) - 1)
'cKeyValue = Trim(cKeyValue)

SetIniValue = iRetval
End Function


'@desc      Validates the correctness of a template by reading it's template.ini.
'           It reads a value from an INI file and returns that value.
'@param     cTemplateFolder     The folder in which the template is stored. This is the full path.
'@param     cErrorString        Any user friendly error message is returned by reference in this. This error message should be displayed, if this function returns false.
'@param     bSCORM              Flag to indicate if SCORM specific settings need to be checked.
'@return    ValidateTemplate    Returns true, if the template is valid.
Public Function ValidateTemplate(ByVal cTemplateFolder As String, _
                                    ByRef cErrorString As String, _
                                    ByVal bSCORM As Boolean) As Boolean
ValidateTemplate = True

Dim oFSO As New FileSystemObject

Dim cTemplateIniFile As String
cTemplateIniFile = cTemplateFolder & "\template.ini"

Dim cListingFile As String
cListingFile = GetIniValue(cTemplateIniFile, "listing", "templatefile", "")

If cListingFile = "" Then
    cErrorString = "Please specify a 'listing' template file in template.ini."
    ValidateTemplate = False
    Exit Function
End If

If oFSO.FileExists(cTemplateFolder & "\" & cListingFile) = False Then
    cErrorString = "The specified 'listing' template file (" & cListingFile & ") in template.ini does not exist."
    ValidateTemplate = False
    Exit Function
End If

Dim cFramesetFile As String
cFramesetFile = GetIniValue(cTemplateFolder & "\template.ini", "scorm", "framesetfile", "")

If bSCORM = True And cFramesetFile = "" Then
    cErrorString = "There is no 'frameset' file in the [scorm] section."
    ValidateTemplate = False
    Exit Function
End If

'This is not compulsory
If cFramesetFile <> "" Then
    If oFSO.FileExists(cTemplateFolder & "\" & cFramesetFile) = False Then
        cErrorString = "The specified 'frameset' file (" & cFramesetFile & ") in the [scorm] section, in template.ini does not exist."
        ValidateTemplate = False
        Exit Function
    End If
End If

Dim cDefLayout As String
cDefLayout = GetIniValue(cTemplateFolder & "\template.ini", "general", "defaultlayout", "")

If cDefLayout = "" Then
    cErrorString = "Please specify a default layout in template.ini."
    ValidateTemplate = False
    Exit Function
End If

Dim cDefLayoutFile As String
cDefLayoutFile = GetIniValue(cTemplateFolder & "\template.ini", cDefLayout, "templatefile", "")

If cDefLayoutFile = "" Then
    cErrorString = "Please specify a templatefile for the default layout."
    ValidateTemplate = False
    Exit Function
End If

If oFSO.FileExists(cTemplateFolder & "\" & cDefLayoutFile) = False Then
    cErrorString = "The templatefile for the default layout in template.ini (" & cDefLayoutFile & ") does not exist."
    ValidateTemplate = False
    Exit Function
End If


End Function

'@desc      This is a central function which is used to generate physical filenames for all types of objects.
'           You can set the file naming conventions here.
'@param     cName               The name of the object.
'@param     cExtension          The extension of the file name.
'@param     cType               The type of object e.g. "PAGE". This should be all caps.
'@param     cID                 The unique ID of the object.
'@return    GeneratePageName    Returns the generated page name.
'@comments  [TODO]Include a user interface where the user can choose her own naming convention.
'           [TODO]Generate page names in m1_s1_p1.ext format, which is user defined.
'           Make sure that any no. of levels can be accomodated.

 Function GeneratePageName(cName As String, cExtension As String, cType As String, cID As Long) As String
If cType = "COURSE" Then
    GeneratePageName = "home" & cExtension
Else
    'GeneratePageName = cName & "_" & Mid(LCase(cType), 1, 1) & cID & cExtension
    'GeneratePageName = Mid(LCase(cType), 1, 1) & cID & cExtension
    GeneratePageName = LCase(cType) & cID & cExtension
End If
End Function

'@desc      Encode a string in html format for use in HTML or XML.
'@param     StringToEncode                  The string to be encoded.
'@param     UsePlusRatherThanHexForSpace    Specifies which character(s) is to be used for spaces.
'@return    URLEncode           Returns the encoded string.
Public Function URLEncode(StringToEncode As String, Optional _
   UsePlusRatherThanHexForSpace As Boolean = False) As String

Dim TempAns As String
Dim CurChr As Integer
CurChr = 1
Do Until CurChr - 1 = Len(StringToEncode)
  Select Case Asc(Mid(StringToEncode, CurChr, 1))
    Case 48 To 57, 65 To 90, 97 To 122
      TempAns = TempAns & Mid(StringToEncode, CurChr, 1)
    Case 32
      If UsePlusRatherThanHexForSpace = True Then
        TempAns = TempAns & "+"
      Else
        TempAns = TempAns & "%" & Hex(32)
      End If
   Case Else
         TempAns = TempAns & "%" & _
              Format(Hex(Asc(Mid(StringToEncode, _
              CurChr, 1))), "00")
End Select

  CurChr = CurChr + 1
Loop

URLEncode = TempAns
End Function

'@desc      Decodes a string in html format to regular string.
'@param     StringToDecode  The string to be decoded.
'@return    URLDecode       Returns the decoded string.
Public Function URLDecode(StringToDecode As String) As String

Dim TempAns As String
Dim CurChr As Integer

CurChr = 1

Do Until CurChr - 1 = Len(StringToDecode)
  Select Case Mid(StringToDecode, CurChr, 1)
    Case "+"
      TempAns = TempAns & " "
    Case "%"
      TempAns = TempAns & Chr(Val("&h" & _
         Mid(StringToDecode, CurChr + 1, 2)))
       CurChr = CurChr + 2
    Case Else
      TempAns = TempAns & Mid(StringToDecode, CurChr, 1)
  End Select

CurChr = CurChr + 1
Loop

URLDecode = TempAns
End Function

Public Function CreateTag(ByVal oSettings As Settings, _
                                Optional cPreviousTag As String) As String
    Dim oSetting As Setting
    Dim cTag As String
    cTag = cPreviousTag
    For Each oSetting In oSettings
        If cTag = "" Then
            cTag = oSetting.Key & cKeyValueSeperator & oSetting.Value
        Else
            cTag = cTag + cPairSeperator + oSetting.Key & cKeyValueSeperator & oSetting.Value
        End If
    Next
    CreateTag = cTag
End Function

Public Function RetrieveTag(ByVal cTag As String, _
                            ByRef oSettings As Settings)
    
    'name=anand::age=23::sex=m
    Dim iPos As Integer
    iPos = 1
    
    Dim cKeyValue As String
    Dim cKey As String
    Dim cValue As String
    Dim iKeyPos As Integer
    
    Do While iPos <> 0
        iPos = InStr(cTag, cPairSeperator)
        If iPos = 0 Then
            iKeyPos = InStr(cTag, cKeyValueSeperator)
            cKey = Mid(cTag, 1, iKeyPos - 1)
            cValue = Mid(cTag, iKeyPos + 1)
            oSettings.Add cKey, cValue, cKey
            Exit Do
        End If
        cKeyValue = Mid(cTag, 1, iPos - 1)
        cTag = Mid(cTag, iPos + Len(cPairSeperator))
        
        iKeyPos = InStr(cKeyValue, cKeyValueSeperator)
        cKey = Mid(cKeyValue, 1, iKeyPos - 1)
        cValue = Mid(cKeyValue, iKeyPos + 1)
        
        oSettings.Add cKey, cValue, cKey
    Loop
End Function

Public Function SparseTag(sSearched As String, sSought As String, sKeySeperator As String, sTagSeperator As String) As String
    Dim iStartSearch As Integer
    Dim sEndSearch As Integer
    'MsgBox sSearched
    iStartSearch = InStr(sSearched, sSought)
    'MsgBox sSearched
    If iStartSearch = 0 Then
        SparseTag = 0
    Else
        iStartSearch = InStr(iStartSearch, sSearched, sKeySeperator) + 1
        sEndSearch = InStr(iStartSearch, sSearched, sTagSeperator)
        If sEndSearch = 0 Then
            SparseTag = Mid(sSearched, iStartSearch)
        Else
            SparseTag = Mid(sSearched, iStartSearch, (sEndSearch - iStartSearch))
        End If
    End If
End Function

Public Sub LaunchFile(cFile As String)
If cFile = "" Then Exit Sub
Dim iExecuteRetVal As Long
iExecuteRetVal = ShellExecute(0, "Edit with Dreamweaver", cFile, 0, 0, 1)
End Sub

Public Function GenHtmlFromXML(cXMLFile As String, cXSLFile As String) As String
Dim objXML, objXSL
'objXML.async = False

Set objXML = CreateObject("microsoft.xmldom")
Set objXSL = CreateObject("microsoft.xmldom")

'Dim cXMLFile As String
'Dim cXSLFile As String
'Dim cOutputHTMLFile As String

'cXMLFile = Text1.Text
'cXSLFile = Text2.Text
'cOutputHTMLFile = Text3.Text

Dim transformedHTML As String ', displayHTML As String

objXML.Load cXMLFile

'objXSL.async = False
objXSL.Load cXSLFile
transformedHTML = objXML.transformNode(objXSL.documentElement)


GenHtmlFromXML = transformedHTML

'Dim oFSO As New FileSystemObject
'Dim oTextStream As TextStream
'Set oTextStream = oFSO.CreateTextFile(cOutputHTMLFile, True)
'oTextStream.Write transformedHTML
'oTextStream.Close

'displayHTML = objXSL.transformNode(objDefaultXSL.documentElement)


'displayFile = Request.QueryString("xml")
'If IsEmpty(displayFile) Then
'displayFile = "XSL Stylesheet:" & XSLfile
'displayHTML = objXSL.transformNode(objDefaultXSL.documentElement)
'Else
'displayFile = "XML file:" & XMLfile
'displayHTML = "objXML.transformNode(objDefaultXSL.documentElement)"
'End If

'MsgBox "Done!", vbInformation
'MsgBox displayHTML

End Function

